package com.kampei.ui
{
	import flash.display.MovieClip;
	
	public class FSliderClass extends FUIComponentClass
	{
		private var depth:Number = 0;
		
		private var minValue:Number = 0;
		private var maxValue:Number = 100;
		private var range:Number = 100;
		private var value:Number = 0;
		
		private var tickFrequency:Number = 2;
		private var tickJumpSize:Number = 10;
		private var tickHeight:Number = 2;
		private var tickSpacing:Number = 3;
		private var tickStyle:String = "none";
		private var tickSnapping:Boolean = false;
		
		private var sm_stepAmount:Number = 1;
		private var lg_stepAmount:Number = 5;
		private var horizontal:Boolean = true;
		private var thumbSymbol:String = "fsl_squareThumb";
		private var fillColor:Number = 0xCCCCCC;

		public function FSliderClass(){
			Width = (Horizontal) ? width : height;
			super();
			scaleX = scaleY = 100;
			rotation = 0;
			
			attachMovie("fsl_scrollTrack", "scrollTrack_mc", depth++, {y:1});
			setThumbSymbol(thumbSymbol );
			setEnabled( this.enabled );
			
			if(minValue >= maxValue){
				minValue = 0;
				maxValue = 100;
			}
		
			setSliderProperties( minValue, maxValue );
			setTickFrequency(tickFrequency );
			
			setStyleProperty("fillFace", fillColor, true);
			delete fillColor;
			
			setSize(this.Width);
			setHorizontal(horizontal );	
			
			setChangeHandler(changeHandler);
		}
	}
}

/* Getters */
public function get minValue():Number{
	return minValue;
}

// getSnapToTicks
public function get SnapToTicks():Number{
	return tickSnapping;
}

public function get valueInt():int{
	return Math.round(value);
}

public function get tickStyle():String{
	return tickStyle;
}

public function get thumbSymbol():String{
	return thumbSymbol;
}

public function get horizontal():Number{
	return horizontal;
}

public function get tickFrequency():Number{
	return tickFrequency;
}

public function get smallStepAmount():Number{
	return sm_stepAmount;
}

public function get largeStepAmount():Number{
	return lg_stepAmount;
}

public function get maxValue():Number{
	return maxValue;
}

public function get value():Number{
	return this.value;
}

/* Setters */
public function set size(w:Number):void{
	if( width < 4 ) return;
	
	width = w;
	
	scrollableArea = (thumb_mc) ? width-2-thumb_mc.width : width - 5;
	scrollTrack_mc.x = width/2;

	this.drawTicks();
	this.setValue(this.value);
	
}

FSliderClass.prototype.setSnapToTicks = function( snap ){
	this.tickSnapping = snap;
	this.setValue(this.value);
}

FSliderClass.prototype.setSliderProperties = function ( min, max ){
	if(max <= min) return false;

	this.maxValue = Number(max);
	this.minValue = Number(min);
	
	if( isNaN(this.maxValue) || isNaN(this.minValue) ) return false;
	
	this.range = this.maxValue - this.minValue;
	
	this.setTickFrequency(this.tickFrequency);
	
	this.setValue(this.value);
}

FSliderClass.prototype.setEnabled = function( enable ){
	super.setEnabled(enable);
	this.enabled = enable;
	if(!enable){
		this.scrollTrack_mc.gotoAndStop("disabled");
		this.thumb_mc.gotoAndStop("disabled");
		delete this.onPress;
		delete this.onRelease;
		delete this.onReleaseOutside;
		delete this.onMouseMove;
	}else{
		this.scrollTrack_mc.gotoAndStop(1);
		this.thumb_mc.gotoAndStop(1);
		this.onPress = this._onPress;
		this.onRelease = this.onReleaseOutside = this._onRelease;
	}
	this.setSize(this.width);
}

FSliderClass.prototype.setTickStyle = function( style ){
	this.tickStyle = style;
	this._drawTicks();
}

FSliderClass.prototype.setThumbSymbol = function( symbol ){
	var depth;
	
	this.thumbSymbol = symbol;
	
	if(this.thumb_mc){
		depth = this.thumb_mc.getDepth();
		this.thumb_mc.removeMovieClip();
	}else depth = this.depth++;
	
	this.attachMovie( symbol, "thumb_mc", depth);
	
	this.setSize(this.width);

}

FSliderClass.prototype.setHorizontal=function( horizontal ){
	if (horizontal) {
		this._rotation = 0;
		this._xscale = 100;
		this.scrollTrack_mc._rotation = 0;	
		this.scrollTrack_mc._yscale = 100;		
	} else if (!horizontal) {
		this._rotation = 90;
		this.scrollTrack_mc._rotation = 180;	
		this.scrollTrack_mc._yscale = -100;
	}
	this.horizontal = horizontal;
	this.setValue(this.getValue());	
}

FSliderClass.prototype.setSmallStepAmount = function(step){
	if(step<1){
		return false;
	}
	this.sm_stepAmount = Number(step);
}

FSliderClass.prototype.setLargeStepAmount = function(step){
	if(step<1){
		return false;
	}
	this.lg_stepAmount = Number(step);
}

FSliderClass.prototype.setValue = function ( value ){
	var callHandler = false;

	value = Math.min(Math.max(value,this.minValue),this.maxValue);
	
	if(this.tickSnapping) value = this._snapValue(value);
	
	if(this.value != value) callHandler = true;
	
	this.value = value;
	var xPos = this.valueToCoord( value );
	this.thumb_mc._x = xPos;

	this.scrollTrack_mc.left_mc._x = -this.width/2; 
	
	var rightElement = this.scrollTrack_mc.middleRight_mc;
	var leftElement = this.scrollTrack_mc.middleLeft_mc;		

	if(!this.horizontal){
		xPos = this.valueToCoord((this.range-value+this.minValue*2));
	}
	
	leftElement._width = xPos + this.thumb_mc._width/2;
	leftElement._x = this.scrollTrack_mc.left_mc._x+this.scrollTrack_mc.left_mc._width;
	
	rightElement._width = this.width - xPos - this.scrollTrack_mc.left_mc._width - this.scrollTrack_mc.right_mc._width - this.thumb_mc._width/2;
	rightElement._x = leftElement._x + leftElement._width;
	
	this.scrollTrack_mc.right_mc._x = rightElement._x + rightElement._width;

	if(callHandler) this.executeCallBack();
}

FSliderClass.prototype.setTickFrequency = function( frequency ){
	frequency = Number(frequency);
	if(isNaN(frequency)) return false;
	frequency = Math.max(frequency, 2);
	this.tickFrequency = frequency;
	this.tickJumpSize = this.range / (this.tickFrequency-1);
	this._drawTicks();
	this.setValue(this.value);
}

/* Private */
FSliderClass.prototype._onMouseMove = function(){
	this.setValueFromThumb();
}

FSliderClass.prototype._onRelease = function(){
	this.setValueFromThumb();
	delete this.onMouseMove;
	delete this.thumbPressed;
	delete this.distanceFromCenter;
	this.thumb_mc.gotoAndStop(1);
}

FSliderClass.prototype._snapValue = function( value ){
		return Math.round((value-this.minValue)/this.tickJumpSize)*this.tickJumpSize + this.minValue;
}

FSliderClass.prototype.getValueFromThumb = function(){
	var xMouse;
	if(!this.thumbPressed){
		xMouse = this._xmouse-this.thumb_mc._width/2;
	}else{
		xMouse = this._xmouse - this.distanceFromCenter;
	}
	xMouse = Math.max(1,Math.min(xMouse,this.scrollableArea+1));
	return this.coordToValue(xMouse);
}

FSliderClass.prototype._onPress = function(){
	this.pressFocus();

	this.thumbPressed = this.thumb_mc.hitTest(_root._xmouse, _root._ymouse);

	if(this.thumbPressed){
		this.distanceFromCenter = this._xmouse-this.thumb_mc._x;
	}
	this.setValueFromThumb();
	this.onMouseMove = this._onMouseMove;
	this.thumb_mc.gotoAndStop("pressed");
}


FSliderClass.prototype.setValueFromThumb = function( ){
	this.setValue(this.getValueFromThumb());
}

FSliderClass.prototype.coordToValue = function( coord ){
	if(this.horizontal){
		var value = (coord-1) / (this.scrollableArea) * this.range + this.minValue;
	}else{
		var value = (Math.abs(coord-1-this.scrollableArea) / this.scrollableArea) * this.range + this.minValue;
	}

	return value;
}

FSliderClass.prototype.valueToCoord = function( value ){
	if(this.horizontal){
		return ((value - this.minValue) / this.range * (this.scrollableArea))+1;
	}else{
		return (Math.abs(value-(this.range+this.minValue)) / this.range * (this.scrollableArea))+1;
	}
}

FSliderClass.prototype._drawTicks = function(){
	var depth;
	if(!this.tick_mc){
		this.createEmptyMovieClip("tick_mc", this.depth++);
		this.registerSkinElement(this.tick_mc, "tickColor");
	}
	this.tick_mc.clear();
	var curValue = this.minValue;
	var y = this.scrollTrack_mc._height + this.scrollTrack_mc._y + this.tickSpacing;
	for(var i = 0; i<this.tickFrequency; i++){
		var x = this.valueToCoord(curValue);
		x+=this.thumb_mc._width/2;
		curValue+=this.tickJumpSize;
		if(this.tickStyle == "bottom" || this.tickStyle=="both"){
			with(this.tick_mc){
				lineStyle(0, 0x000000);
				moveTo(x,y);
				lineTo(x, y+this.tickHeight);
			}
		}
		if(this.tickStyle == "top" || this.tickStyle=="both"){
			with(this.tick_mc){
				lineStyle(0, 0x000000);
				moveTo(x,-this.tickSpacing+1);
				lineTo(x, -this.tickSpacing-this.tickHeight+1);
			}
		}

	}
}

FSliderClass.prototype.myOnKeyDown = function(){
	if(!this.enable) return;

	var increase = (!this.horizontal) ? Key.UP : Key.RIGHT;
	var decrease = (!this.horizontal) ? Key.DOWN : Key.LEFT;
	
	var code = Key.getCode();
	var step = (Key.isDown(Key.CONTROL)) ? this.lg_stepAmount : this.sm_stepAmount;
	
	if(this.tickSnapping && step < this.tickJumpSize) step = this.tickJumpSize;
	
	if(code == increase) {
		this.setValue(this.value+step);
	}else if(code==decrease){
		this.setValue(this.value - step);
	}
}
#endinitclip
